import Matrix4 from '@hai2007/tool/Matrix4.js';
import { isNumber, isArray } from '@hai2007/tool/type';

/**
 * 照相机
 * -------------------------
 */
export default function (CORE, CONFIG) {

    return function (options = {}) {

        return new function Camera() {

            // 压缩空间的范围
            if (isNumber(options.size)) {
                options.size = [options.size, options.size, options.size];
            } else if (!isArray(options.size)) {
                options.size = [1, 1, 1];
            }

            // 摄像头位置改变和物体位置改变矩阵初始化
            let matrix4 = Matrix4();

            // 应用投影矩阵
            if ('perspective' in options) {
                let d = options.perspective - 1;
                matrix4.multiply([
                    d + 2, 0, 0, 0,
                    0, d + 2, 0, 0,
                    0, 0, 1, -1,
                    0, 0, -1 * (d + 1), d + 1
                ]);
            }

            // 应用压缩空间矩阵
            matrix4.multiply([
                1 / options.size[0], 0, 0, 0,
                0, 1 / options.size[1], 0, 0,
                0, 0, 1 / options.size[2], 0,
                0, 0, 0, 1
            ]);

            /**
             * 摄像头位置改变
             */

            // 旋转
            this.rotateEye = (deg, a1, b1, c1, a2, b2, c2) => {
                matrix4.rotate(-deg, a1, b1, c1, a2, b2, c2);
                return this;
            };

            // 移动
            this.moveEye = (dis, a, b, c) => {
                matrix4.move(-dis, a, b, c);
                return this;
            };

            /**
             * 物体位置改变
             */

            // 旋转
            this.rotateBody = (deg, a1, b1, c1, a2, b2, c2) => {
                matrix4.rotate(deg, a1, b1, c1, a2, b2, c2);
                return this;
            };

            // 移动
            this.moveBody = (dis, a, b, c) => {
                matrix4.move(dis, a, b, c);
                return this;
            };

            // 缩放
            this.scaleBody = (xTimes, yTimes, zTimes, cx, cy, cz) => {
                matrix4.scale(xTimes, yTimes, zTimes, cx, cy, cz);
                return this;
            };

            // 获取当前的变换矩阵值
            this.value = () => matrix4.value();

        };

    };

};
